@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE examples.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    The code included in this file is provided under the terms of the ISC license
    http://www.isc.org/downloads/software-support-policy/isc-license. Permission
@@ -65,7 +65,7 @@ struct MidiDeviceListEntry : ReferenceCountedObject
 //==============================================================================
 class MidiDemo  : public Component,
                   private Timer,
-                  private MidiKeyboardStateListener,
+                  private MidiKeyboardState::Listener,
                   private MidiInputCallback,
                   private AsyncUpdater
 {
@@ -273,7 +273,7 @@ private:
 
 
             g.setColour (textColour);
-            g.setFont (height * 0.7f);
+            g.setFont ((float) height * 0.7f);
 
             if (isInput)
             {
