@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -97,7 +97,7 @@ void MPEChannelAssigner::noteOff (int noteNumber, int midiChannel)
         return false;
     };
 
-    if (midiChannel >= 0 && midiChannel < 17)
+    if (midiChannel >= 0 && midiChannel <= 16)
     {
         removeNote (midiChannels[midiChannel], noteNumber);
         return;
