@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -24,11 +23,12 @@
   ==============================================================================
 */
 
+#include <juce_core/system/juce_TargetPlatform.h>
+
 #if JucePlugin_Build_Unity
 
-#include "../../juce_core/system/juce_TargetPlatform.h"
 #include "../utility/juce_IncludeModuleHeaders.h"
-#include "../../juce_audio_processors/format_types/juce_LegacyAudioParameter.cpp"
+#include <juce_audio_processors/format_types/juce_LegacyAudioParameter.cpp>
 
 #if JUCE_WINDOWS
  #include "../utility/juce_IncludeSystemHeaders.h"
@@ -60,6 +60,9 @@ public:
     Point<float> localToGlobal (Point<float> relativePosition) override    { return relativePosition + getBounds().getPosition().toFloat(); }
     Point<float> globalToLocal (Point<float> screenPosition) override      { return screenPosition - getBounds().getPosition().toFloat(); }
 
+    using ComponentPeer::localToGlobal;
+    using ComponentPeer::globalToLocal;
+
     StringArray getAvailableRenderingEngines() override                    { return StringArray ("Software Renderer"); }
 
     void setBounds (const Rectangle<int>& newBounds, bool) override
@@ -322,7 +325,7 @@ public:
         short configs[][2] = { JucePlugin_PreferredChannelConfigurations };
         const int numConfigs = sizeof (configs) / sizeof (short[2]);
 
-        jassert (numConfigs > 0 && (configs[0][0] > 0 || configs[0][1] > 0));
+        jassertquiet (numConfigs > 0 && (configs[0][0] > 0 || configs[0][1] > 0));
 
         pluginInstance->setPlayConfigDetails (configs[0][0], configs[0][1], state->sampleRate, samplesPerBlock);
        #else
@@ -362,7 +365,7 @@ public:
 
         if (parametersPtr == nullptr)
         {
-            numParams = juceParameters.params.size();
+            numParams = (int) juceParameters.size();
 
             parametersPtr.reset (static_cast<UnityAudioParameterDefinition*> (std::calloc (static_cast<size_t> (numParams),
                                                                               sizeof (UnityAudioParameterDefinition))));
@@ -371,13 +374,16 @@ public:
 
             for (int i = 0; i < numParams; ++i)
             {
-                auto* parameter = juceParameters.params[i];
+                auto* parameter = juceParameters.getParamForIndex (i);
                 auto& paramDef = parametersPtr.get()[i];
 
-                strncpy (paramDef.name, parameter->getName (15).toRawUTF8(), 15);
+                const auto nameLength = (size_t) numElementsInArray (paramDef.name);
+                const auto unitLength = (size_t) numElementsInArray (paramDef.unit);
+
+                parameter->getName ((int) nameLength - 1).copyToUTF8 (paramDef.name, nameLength);
 
                 if (parameter->getLabel().isNotEmpty())
-                    strncpy (paramDef.unit, parameter->getLabel().toRawUTF8(), 15);
+                    parameter->getLabel().copyToUTF8 (paramDef.unit, unitLength);
 
                 parameterDescriptions.add (parameter->getName (15));
                 paramDef.description = parameterDescriptions[i].toRawUTF8();
@@ -552,7 +558,7 @@ namespace UnityCallbacks
         auto* pluginInstance = state->getEffectData<AudioProcessorUnityWrapper>();
         *value = pluginInstance->getParameter (index);
 
-        strncpy (valueStr, pluginInstance->getParameterString (index).toRawUTF8(), 15);
+        pluginInstance->getParameterString (index).copyToUTF8 (valueStr, 15);
 
         return 0;
     }
@@ -636,7 +642,7 @@ static void declareEffect (UnityAudioEffectDefinition& definition)
     if (! name.startsWithIgnoreCase ("audioplugin"))
         name = "audioplugin_" + name;
 
-    strcpy (definition.name, name.toRawUTF8());
+    name.copyToUTF8 (definition.name, (size_t) numElementsInArray (definition.name));
 
     definition.structSize = sizeof (UnityAudioEffectDefinition);
     definition.parameterStructSize = sizeof (UnityAudioParameterDefinition);
