@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -23,7 +23,7 @@
 namespace juce
 {
 
-#if ! JUCE_MINGW
+#if JUCE_MSVC && ! defined (__INTEL_COMPILER)
  #pragma intrinsic (__cpuid)
  #pragma intrinsic (__rdtsc)
 #endif
@@ -41,10 +41,11 @@ void Logger::outputDebugString (const String& text)
 
 //==============================================================================
 
-#if JUCE_MINGW
+#if JUCE_MINGW || JUCE_CLANG
 static void callCPUID (int result[4], uint32 type)
 {
-  uint32 la = result[0], lb = result[1], lc = result[2], ld = result[3];
+  uint32 la = (uint32) result[0], lb = (uint32) result[1],
+         lc = (uint32) result[2], ld = (uint32) result[3];
 
   asm ("mov %%ebx, %%esi \n\t"
        "cpuid \n\t"
@@ -55,16 +56,13 @@ static void callCPUID (int result[4], uint32 type)
         #endif
        );
 
-  result[0] = la; result[1] = lb; result[2] = lc; result[3] = ld;
+  result[0] = (int) la; result[1] = (int) lb;
+  result[2] = (int) lc; result[3] = (int) ld;
 }
 #else
 static void callCPUID (int result[4], int infoType)
 {
-   #if JUCE_PROJUCER_LIVE_BUILD
-    std::fill (result, result + 4, 0);
-   #else
     __cpuid (result, infoType);
-   #endif
 }
 #endif
 
@@ -148,24 +146,27 @@ void CPUInformation::initialise() noexcept
     hasSSSE3 = (info[2] & (1 <<  9)) != 0;
     hasSSE41 = (info[2] & (1 << 19)) != 0;
     hasSSE42 = (info[2] & (1 << 20)) != 0;
+
+    JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wshift-sign-overflow")
     has3DNow = (info[1] & (1 << 31)) != 0;
+    JUCE_END_IGNORE_WARNINGS_GCC_LIKE
 
     callCPUID (info, 0x80000001);
     hasFMA4  = (info[2] & (1 << 16)) != 0;
 
     callCPUID (info, 7);
 
-    hasAVX2            = (info[1] & (1 << 5))   != 0;
-    hasAVX512F         = (info[1] & (1u << 16)) != 0;
-    hasAVX512DQ        = (info[1] & (1u << 17)) != 0;
-    hasAVX512IFMA      = (info[1] & (1u << 21)) != 0;
-    hasAVX512PF        = (info[1] & (1u << 26)) != 0;
-    hasAVX512ER        = (info[1] & (1u << 27)) != 0;
-    hasAVX512CD        = (info[1] & (1u << 28)) != 0;
-    hasAVX512BW        = (info[1] & (1u << 30)) != 0;
-    hasAVX512VL        = (info[1] & (1u << 31)) != 0;
-    hasAVX512VBMI      = (info[2] & (1u <<  1)) != 0;
-    hasAVX512VPOPCNTDQ = (info[2] & (1u << 14)) != 0;
+    hasAVX2            = ((unsigned int) info[1] & (1 << 5))   != 0;
+    hasAVX512F         = ((unsigned int) info[1] & (1u << 16)) != 0;
+    hasAVX512DQ        = ((unsigned int) info[1] & (1u << 17)) != 0;
+    hasAVX512IFMA      = ((unsigned int) info[1] & (1u << 21)) != 0;
+    hasAVX512PF        = ((unsigned int) info[1] & (1u << 26)) != 0;
+    hasAVX512ER        = ((unsigned int) info[1] & (1u << 27)) != 0;
+    hasAVX512CD        = ((unsigned int) info[1] & (1u << 28)) != 0;
+    hasAVX512BW        = ((unsigned int) info[1] & (1u << 30)) != 0;
+    hasAVX512VL        = ((unsigned int) info[1] & (1u << 31)) != 0;
+    hasAVX512VBMI      = ((unsigned int) info[2] & (1u <<  1)) != 0;
+    hasAVX512VPOPCNTDQ = ((unsigned int) info[2] & (1u << 14)) != 0;
 
     SYSTEM_INFO systemInfo;
     GetNativeSystemInfo (&systemInfo);
@@ -215,7 +216,7 @@ static DebugFlagsInitialiser debugFlagsInitialiser;
 #else
  RTL_OSVERSIONINFOW getWindowsVersionInfo()
  {
-     RTL_OSVERSIONINFOW versionInfo = { 0 };
+     RTL_OSVERSIONINFOW versionInfo = {};
 
      if (auto* moduleHandle = ::GetModuleHandleW (L"ntdll.dll"))
      {
@@ -227,7 +228,7 @@ static DebugFlagsInitialiser debugFlagsInitialiser;
              LONG STATUS_SUCCESS = 0;
 
              if (rtlGetVersion (&versionInfo) != STATUS_SUCCESS)
-                 versionInfo = { 0 };
+                 versionInfo = {};
          }
      }
 
@@ -274,6 +275,27 @@ String SystemStats::getOperatingSystemName()
         case WinVista:          name = "Windows Vista";     break;
         case WinXP:             name = "Windows XP";        break;
         case Win2000:           name = "Windows 2000";      break;
+
+        case MacOSX:            JUCE_FALLTHROUGH
+        case Windows:           JUCE_FALLTHROUGH
+        case Linux:             JUCE_FALLTHROUGH
+        case Android:           JUCE_FALLTHROUGH
+        case iOS:               JUCE_FALLTHROUGH
+
+        case MacOSX_10_7:       JUCE_FALLTHROUGH
+        case MacOSX_10_8:       JUCE_FALLTHROUGH
+        case MacOSX_10_9:       JUCE_FALLTHROUGH
+        case MacOSX_10_10:      JUCE_FALLTHROUGH
+        case MacOSX_10_11:      JUCE_FALLTHROUGH
+        case MacOSX_10_12:      JUCE_FALLTHROUGH
+        case MacOSX_10_13:      JUCE_FALLTHROUGH
+        case MacOSX_10_14:      JUCE_FALLTHROUGH
+        case MacOSX_10_15:      JUCE_FALLTHROUGH
+        case MacOS_11:          JUCE_FALLTHROUGH
+        case MacOS_12:          JUCE_FALLTHROUGH
+
+        case UnknownOS:         JUCE_FALLTHROUGH
+        case WASM:              JUCE_FALLTHROUGH
         default:                jassertfalse; break; // !! new type of OS?
     }
 
@@ -309,8 +331,16 @@ bool SystemStats::isOperatingSystem64Bit()
    #else
     typedef BOOL (WINAPI* LPFN_ISWOW64PROCESS) (HANDLE, PBOOL);
 
+    const auto moduleHandle = GetModuleHandleA ("kernel32");
+
+    if (moduleHandle == nullptr)
+    {
+        jassertfalse;
+        return false;
+    }
+
     LPFN_ISWOW64PROCESS fnIsWow64Process
-        = (LPFN_ISWOW64PROCESS) GetProcAddress (GetModuleHandleA ("kernel32"), "IsWow64Process");
+        = (LPFN_ISWOW64PROCESS) GetProcAddress (moduleHandle, "IsWow64Process");
 
     BOOL isWow64 = FALSE;
 
@@ -378,7 +408,7 @@ public:
         LARGE_INTEGER f;
         QueryPerformanceFrequency (&f);
         hiResTicksPerSecond = f.QuadPart;
-        hiResTicksScaleFactor = 1000.0 / hiResTicksPerSecond;
+        hiResTicksScaleFactor = 1000.0 / (double) hiResTicksPerSecond;
     }
 
     inline int64 getHighResolutionTicks() noexcept
@@ -390,7 +420,7 @@ public:
 
     inline double getMillisecondCounterHiRes() noexcept
     {
-        return getHighResolutionTicks() * hiResTicksScaleFactor;
+        return (double) getHighResolutionTicks() * hiResTicksScaleFactor;
     }
 
     int64 hiResTicksPerSecond, hiResTicksOffset;
@@ -525,20 +555,36 @@ String SystemStats::getUserRegion()       { return getLocaleValue (LOCALE_USER_D
 String SystemStats::getDisplayLanguage()
 {
     DynamicLibrary dll ("kernel32.dll");
-    JUCE_LOAD_WINAPI_FUNCTION (dll, GetUserDefaultUILanguage, getUserDefaultUILanguage, LANGID, (void))
+    JUCE_LOAD_WINAPI_FUNCTION (dll,
+                               GetUserPreferredUILanguages,
+                               getUserPreferredUILanguages,
+                               BOOL,
+                               (DWORD, PULONG, PZZWSTR, PULONG))
 
-    if (getUserDefaultUILanguage == nullptr)
-        return "en";
+    constexpr auto defaultResult = "en";
 
-    auto langID = MAKELCID (getUserDefaultUILanguage(), SORT_DEFAULT);
+    if (getUserPreferredUILanguages == nullptr)
+        return defaultResult;
 
-    auto mainLang = getLocaleValue (langID, LOCALE_SISO639LANGNAME, "en");
-    auto region   = getLocaleValue (langID, LOCALE_SISO3166CTRYNAME, nullptr);
+    ULONG numLanguages = 0;
+    ULONG numCharsInLanguagesBuffer = 0;
 
-    if (region.isNotEmpty())
-        mainLang << '-' << region;
+    // Retrieving the necessary buffer size for storing the list of languages
+    if (! getUserPreferredUILanguages (MUI_LANGUAGE_NAME, &numLanguages, nullptr, &numCharsInLanguagesBuffer))
+        return defaultResult;
 
-    return mainLang;
+    std::vector<WCHAR> languagesBuffer (numCharsInLanguagesBuffer);
+    const auto success = getUserPreferredUILanguages (MUI_LANGUAGE_NAME,
+                                                      &numLanguages,
+                                                      languagesBuffer.data(),
+                                                      &numCharsInLanguagesBuffer);
+
+    if (! success || numLanguages == 0)
+        return defaultResult;
+
+    // The buffer contains a zero delimited list of languages, the first being
+    // the currently displayed language.
+    return languagesBuffer.data();
 }
 
 } // namespace juce
