@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -97,12 +96,12 @@ namespace TTFNameExtractor
     {
         input.setPosition (directoryOffset);
 
-        NamingTable namingTable = { 0 };
+        NamingTable namingTable = {};
         input.read (&namingTable, sizeof (namingTable));
 
         for (int i = 0; i < (int) ByteOrder::swapIfLittleEndian (namingTable.numberOfNameRecords); ++i)
         {
-            NameRecord nameRecord = { 0 };
+            NameRecord nameRecord = {};
             input.read (&nameRecord, sizeof (nameRecord));
 
             if (ByteOrder::swapIfLittleEndian (nameRecord.nameID) == 4)
@@ -120,7 +119,7 @@ namespace TTFNameExtractor
 
     static String getTypefaceNameFromFile (MemoryInputStream& input)
     {
-        OffsetTable offsetTable = { 0 };
+        OffsetTable offsetTable = {};
         input.read (&offsetTable, sizeof (offsetTable));
 
         for (int i = 0; i < (int) ByteOrder::swapIfLittleEndian (offsetTable.numTables); ++i)
@@ -154,7 +153,7 @@ namespace FontEnumerators
     {
         if (lpelfe != nullptr && (type & RASTER_FONTTYPE) == 0)
         {
-            LOGFONTW lf = { 0 };
+            LOGFONTW lf = {};
             lf.lfWeight = FW_DONTCARE;
             lf.lfOutPrecision = OUT_OUTLINE_PRECIS;
             lf.lfQuality = DEFAULT_QUALITY;
@@ -165,7 +164,7 @@ namespace FontEnumerators
             const String fontName (lpelfe->elfLogFont.lfFaceName);
             fontName.copyToUTF16 (lf.lfFaceName, sizeof (lf.lfFaceName));
 
-            auto dc = CreateCompatibleDC (0);
+            auto dc = CreateCompatibleDC (nullptr);
             EnumFontFamiliesEx (dc, &lf, (FONTENUMPROCW) &fontEnum2, lParam, 0);
             DeleteDC (dc);
         }
@@ -198,10 +197,10 @@ StringArray Font::findAllTypefaceNames()
     else
    #endif
     {
-        auto dc = CreateCompatibleDC (0);
+        auto dc = CreateCompatibleDC (nullptr);
 
         {
-            LOGFONTW lf = { 0 };
+            LOGFONTW lf = {};
             lf.lfWeight = FW_DONTCARE;
             lf.lfOutPrecision = OUT_OUTLINE_PRECIS;
             lf.lfQuality = DEFAULT_QUALITY;
@@ -301,7 +300,7 @@ Typeface::Ptr Font::getDefaultTypefaceForFont (const Font& font)
     static DefaultFontNames defaultNames;
 
     Font newFont (font);
-    auto& faceName = font.getTypefaceName();
+    auto faceName = font.getTypefaceName();
 
     if (faceName == getDefaultSansSerifFontName())       newFont.setTypefaceName (defaultNames.defaultSans);
     else if (faceName == getDefaultSerifFontName())      newFont.setTypefaceName (defaultNames.defaultSerif);
@@ -340,10 +339,10 @@ public:
         SelectObject (dc, previousFontH); // Replacing the previous font before deleting the DC avoids a warning in BoundsChecker
         DeleteDC (dc);
 
-        if (fontH != 0)
+        if (fontH != nullptr)
             DeleteObject (fontH);
 
-        if (memoryFont != 0)
+        if (memoryFont != nullptr)
             RemoveFontMemResourceEx (memoryFont);
     }
 
@@ -400,17 +399,17 @@ public:
         GLYPHMETRICS gm;
         // (although GetGlyphOutline returns a DWORD, it may be -1 on failure, so treat it as signed int..)
         auto bufSize = (int) GetGlyphOutline (dc, (UINT) glyphNumber, GGO_NATIVE | GGO_GLYPH_INDEX,
-                                              &gm, 0, 0, &identityMatrix);
+                                              &gm, 0, nullptr, &identityMatrix);
 
         if (bufSize > 0)
         {
             HeapBlock<char> data (bufSize);
             GetGlyphOutline (dc, (UINT) glyphNumber, GGO_NATIVE | GGO_GLYPH_INDEX, &gm,
-                             bufSize, data, &identityMatrix);
+                             (DWORD) bufSize, data, &identityMatrix);
 
             auto pheader = reinterpret_cast<const TTPOLYGONHEADER*> (data.getData());
 
-            auto scaleX = 1.0f / tm.tmHeight;
+            auto scaleX = 1.0f / (float) tm.tmHeight;
             auto scaleY = -scaleX;
 
             while ((char*) pheader < data + bufSize)
@@ -418,7 +417,7 @@ public:
                 glyphPath.startNewSubPath (scaleX * pheader->pfxStart.x.value,
                                            scaleY * pheader->pfxStart.y.value);
 
-                auto curve = (const TTPOLYCURVE*) ((const char*) pheader + sizeof (TTPOLYGONHEADER));
+                auto curve = unalignedPointerCast<const TTPOLYCURVE*> ((const char*) pheader + sizeof (TTPOLYGONHEADER));
                 auto curveEnd = ((const char*) pheader) + pheader->cb;
 
                 while ((const char*) curve < curveEnd)
@@ -451,7 +450,7 @@ public:
                     curve = (const TTPOLYCURVE*) &(curve->apfx [curve->cpfx]);
                 }
 
-                pheader = (const TTPOLYGONHEADER*) curve;
+                pheader = unalignedPointerCast<const TTPOLYGONHEADER*> (curve);
 
                 glyphPath.closeSubPath();
             }
@@ -464,7 +463,7 @@ private:
     static const MAT2 identityMatrix;
     HFONT fontH = {};
     HGDIOBJ previousFontH = {};
-    HDC dc { CreateCompatibleDC (0) };
+    HDC dc { CreateCompatibleDC (nullptr) };
     TEXTMETRIC tm;
     HANDLE memoryFont = {};
     float ascent = 1.0f, heightToPointsFactor = 1.0f;
@@ -481,7 +480,7 @@ private:
         SetMapperFlags (dc, 0);
         SetMapMode (dc, MM_TEXT);
 
-        LOGFONTW lf = { 0 };
+        LOGFONTW lf = {};
         lf.lfCharSet = DEFAULT_CHARSET;
         lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
         lf.lfOutPrecision = OUT_OUTLINE_PRECIS;
@@ -494,17 +493,17 @@ private:
 
         auto standardSizedFont = CreateFontIndirect (&lf);
 
-        if (standardSizedFont != 0)
+        if (standardSizedFont != nullptr)
         {
-            if ((previousFontH = SelectObject (dc, standardSizedFont)) != 0)
+            if ((previousFontH = SelectObject (dc, standardSizedFont)) != nullptr)
             {
                 fontH = standardSizedFont;
                 OUTLINETEXTMETRIC otm;
 
                 if (GetOutlineTextMetrics (dc, sizeof (otm), &otm) != 0)
                 {
-                    heightInPoints = otm.otmEMSquare;
-                    lf.lfHeight = -(int) heightInPoints;
+                    heightInPoints = (int) otm.otmEMSquare;
+                    lf.lfHeight = -heightInPoints;
                     fontH = CreateFontIndirect (&lf);
 
                     SelectObject (dc, fontH);
@@ -515,9 +514,9 @@ private:
 
         if (GetTextMetrics (dc, &tm))
         {
-            auto dpi = (GetDeviceCaps (dc, LOGPIXELSX) + GetDeviceCaps (dc, LOGPIXELSY)) / 2.0f;
-            heightToPointsFactor = (dpi / GetDeviceCaps (dc, LOGPIXELSY)) * heightInPoints / (float) tm.tmHeight;
-            ascent = tm.tmAscent / (float) tm.tmHeight;
+            auto dpi = (float) (GetDeviceCaps (dc, LOGPIXELSX) + GetDeviceCaps (dc, LOGPIXELSY)) / 2.0f;
+            heightToPointsFactor = (dpi / (float) GetDeviceCaps (dc, LOGPIXELSY)) * (float) heightInPoints / (float) tm.tmHeight;
+            ascent = (float) tm.tmAscent / (float) tm.tmHeight;
             std::unordered_map<int, int> glyphsForChars;
             defaultGlyph = getGlyphForChar (dc, glyphsForChars, tm.tmDefaultChar);
             createKerningPairs (dc, glyphsForChars, (float) tm.tmHeight);
@@ -527,7 +526,7 @@ private:
     void createKerningPairs (HDC hdc, std::unordered_map<int, int>& glyphsForChars, float height)
     {
         HeapBlock<KERNINGPAIR> rawKerning;
-        auto numKPs = GetKerningPairs (hdc, 0, 0);
+        auto numKPs = GetKerningPairs (hdc, 0, nullptr);
         rawKerning.calloc (numKPs);
         GetKerningPairs (hdc, numKPs, rawKerning);
 
@@ -539,8 +538,8 @@ private:
             auto glyph2 = getGlyphForChar (hdc, glyphsForChars, rawKerning[i].wSecond);
             auto standardWidth = getGlyphWidth (hdc, widthsForGlyphs, glyph1);
 
-            kerningPairs[kerningPairIndex (glyph1, glyph2)] = (standardWidth + rawKerning[i].iKernAmount) / height;
-            kerningPairs[kerningPairIndex (glyph1, -1)]     = standardWidth / height;
+            kerningPairs[kerningPairIndex (glyph1, glyph2)] = (float) (standardWidth + rawKerning[i].iKernAmount) / height;
+            kerningPairs[kerningPairIndex (glyph1, -1)]     = (float) standardWidth / height;
         }
     }
 
@@ -578,7 +577,7 @@ private:
     {
         GLYPHMETRICS gm;
         gm.gmCellIncX = 0;
-        GetGlyphOutline (dc, (UINT) glyphNumber, GGO_NATIVE | GGO_GLYPH_INDEX, &gm, 0, 0, &identityMatrix);
+        GetGlyphOutline (dc, (UINT) glyphNumber, GGO_NATIVE | GGO_GLYPH_INDEX, &gm, 0, nullptr, &identityMatrix);
         return gm.gmCellIncX;
     }
 
@@ -594,7 +593,7 @@ private:
         if (single != kerningPairs.end())
             return single->second;
 
-        auto width = getGlyphWidth (hdc, glyph1) / (float) tm.tmHeight;
+        auto width = (float) getGlyphWidth (hdc, glyph1) / (float) tm.tmHeight;
         kerningPairs[kerningPairIndex (glyph1, -1)] = width;
         return width;
     }
@@ -613,7 +612,7 @@ Typeface::Ptr Typeface::createSystemTypefaceFor (const Font& font)
     {
         std::unique_ptr<WindowsDirectWriteTypeface> wtf (new WindowsDirectWriteTypeface (font, factories->systemFonts));
 
-        if (wtf->loadedOk())
+        if (wtf->loadedOk() && wtf->isFontFound())
             return wtf.release();
     }
    #endif
