@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -66,6 +65,28 @@ private:
 
 #endif
 
+#if JUCE_WINDOWS && JUCE_WIN_PER_MONITOR_DPI_AWARE
+ extern JUCE_API double getScaleFactorForWindow (HWND);
+#endif
+
+static bool contextHasTextureNpotFeature()
+{
+    if (getOpenGLVersion() >= Version (2))
+        return true;
+
+    // If the version is < 2, we can't use the newer extension-checking API
+    // so we have to use glGetString
+    const auto* extensionsBegin = glGetString (GL_EXTENSIONS);
+
+    if (extensionsBegin == nullptr)
+        return false;
+
+    const auto* extensionsEnd = findNullTerminator (extensionsBegin);
+    const std::string extensionsString (extensionsBegin, extensionsEnd);
+    const auto stringTokens = StringArray::fromTokens (extensionsString.c_str(), false);
+    return stringTokens.contains ("GL_ARB_texture_non_power_of_two");
+}
+
 //==============================================================================
 class OpenGLContext::CachedImage  : public CachedComponentImage,
                                     private ThreadPoolJob
@@ -74,7 +95,8 @@ public:
     CachedImage (OpenGLContext& c, Component& comp,
                  const OpenGLPixelFormat& pixFormat, void* contextToShare)
         : ThreadPoolJob ("OpenGL Rendering"),
-          context (c), component (comp)
+          context (c),
+          component (comp)
     {
         nativeContext.reset (new NativeContext (component, pixFormat, contextToShare,
                                                 c.useMultisampling, c.versionRequired));
@@ -95,7 +117,8 @@ public:
     {
         if (nativeContext != nullptr)
         {
-            renderThread.reset (new ThreadPool (1));
+            renderThread = std::make_unique<ThreadPool> (1);
+            renderThread->setThreadPriorities (9);
             resume();
         }
     }
@@ -142,16 +165,6 @@ public:
             renderThread->addJob (this, false);
     }
 
-   #if JUCE_MAC
-    static CVReturn displayLinkCallback (CVDisplayLinkRef, const CVTimeStamp*, const CVTimeStamp*,
-                                         CVOptionFlags, CVOptionFlags*, void* displayLinkContext)
-    {
-        auto* self = (CachedImage*) displayLinkContext;
-        self->renderFrame();
-        return kCVReturnSuccess;
-    }
-   #endif
-
     //==============================================================================
     void paint (Graphics&) override
     {
@@ -290,16 +303,44 @@ public:
 
     void updateViewportSize (bool canTriggerUpdate)
     {
+        JUCE_ASSERT_MESSAGE_THREAD
+
         if (auto* peer = component.getPeer())
         {
-           #if JUCE_WINDOWS
-            auto newScale = nativeContext->getWindowScaleFactor (component.getTopLevelComponent()->getScreenBounds());
+           #if JUCE_MAC
+            const auto displayScale = [this]
+            {
+                if (auto* wrapper = cvDisplayLinkWrapper.get())
+                    if (wrapper->updateActiveDisplay())
+                        nativeContext->setNominalVideoRefreshPeriodS (wrapper->getNominalVideoRefreshPeriodS());
+
+                if (auto* view = getCurrentView())
+                {
+                    if ([view respondsToSelector: @selector (backingScaleFactor)])
+                        return [(id) view backingScaleFactor];
+
+                    if (auto* window = [view window])
+                        return [window backingScaleFactor];
+                }
+
+                return scale;
+            }();
            #else
-            auto newScale = Desktop::getInstance().getDisplays().findDisplayForRect (component.getTopLevelComponent()->getScreenBounds()).scale;
+            const auto displayScale = Desktop::getInstance().getDisplays().getDisplayForRect (component.getTopLevelComponent()->getScreenBounds())->scale;
            #endif
 
             auto localBounds = component.getLocalBounds();
-            auto newArea = peer->getComponent().getLocalArea (&component, localBounds).withZeroOrigin() * newScale;
+            auto newArea = peer->getComponent().getLocalArea (&component, localBounds).withZeroOrigin() * displayScale;
+
+           #if JUCE_WINDOWS && JUCE_WIN_PER_MONITOR_DPI_AWARE
+            auto newScale = getScaleFactorForWindow (nativeContext->getNativeHandle());
+            auto desktopScale = Desktop::getInstance().getGlobalScaleFactor();
+
+            if (! approximatelyEqual (1.0f, desktopScale))
+                newScale *= desktopScale;
+           #else
+            auto newScale = displayScale;
+           #endif
 
             if (scale != newScale || viewportArea != newArea)
             {
@@ -318,10 +359,9 @@ public:
 
     void bindVertexArray() noexcept
     {
-       #if JUCE_OPENGL3
-        if (vertexArrayObject != 0)
-            context.extensions.glBindVertexArray (vertexArrayObject);
-       #endif
+        if (openGLVersion.major >= 3)
+            if (vertexArrayObject != 0)
+                context.extensions.glBindVertexArray (vertexArrayObject);
     }
 
     void checkViewportBounds()
@@ -329,7 +369,10 @@ public:
         auto screenBounds = component.getTopLevelComponent()->getScreenBounds();
 
         if (lastScreenBounds != screenBounds)
+        {
             updateViewportSize (true);
+            lastScreenBounds = screenBounds;
+        }
     }
 
     void paintComponent()
@@ -443,7 +486,7 @@ public:
     JobStatus runJob() override
     {
         {
-            // Allow the message thread to finish setting-up the context before using it..
+            // Allow the message thread to finish setting-up the context before using it.
             MessageManager::Lock::ScopedTryLockType mmLock (messageManagerLock, false);
 
             do
@@ -469,6 +512,7 @@ public:
             if (backgroundProcessCheck.isBackgroundProcess())
             {
                 repaintEvent.wait (300);
+                repaintEvent.reset();
                 continue;
             }
            #endif
@@ -477,13 +521,19 @@ public:
                 break;
 
            #if JUCE_MAC
-            repaintEvent.wait (1000);
-           #else
+            if (context.continuousRepaint)
+            {
+                repaintEvent.wait (-1);
+                renderFrame();
+            }
+            else
+           #endif
             if (! renderFrame())
                 repaintEvent.wait (5); // failed to render, so avoid a tight fail-loop.
             else if (! context.continuousRepaint && ! shouldExit())
                 repaintEvent.wait (-1);
-           #endif
+
+            repaintEvent.reset();
         }
 
         hasInitialised = false;
@@ -496,7 +546,7 @@ public:
 
     bool initialiseOnThread()
     {
-        // On android, this can get called twice, so drop any previous state..
+        // On android, this can get called twice, so drop any previous state.
         associatedObjectNames.clear();
         associatedObjects.clear();
         cachedImageFrameBuffer.release();
@@ -512,17 +562,17 @@ public:
         context.makeActive();
        #endif
 
-        context.extensions.initialise();
+        gl::loadFunctions();
 
-       #if JUCE_OPENGL3
-        if (OpenGLShaderProgram::getLanguageVersion() > 1.2)
+        openGLVersion = getOpenGLVersion();
+
+        if (openGLVersion.major >= 3)
         {
             context.extensions.glGenVertexArrays (1, &vertexArrayObject);
             bindVertexArray();
         }
-       #endif
 
-        glViewport (0, 0, component.getWidth(), component.getHeight());
+        glViewport (0, 0, viewportArea.getWidth(), viewportArea.getHeight());
 
         nativeContext->setSwapInterval (1);
 
@@ -532,13 +582,15 @@ public:
         clearGLError();
        #endif
 
+        textureNpotSupported = contextHasTextureNpotFeature();
+
         if (context.renderer != nullptr)
             context.renderer->newOpenGLContextCreated();
 
        #if JUCE_MAC
-        CVDisplayLinkCreateWithActiveCGDisplays (&displayLink);
-        CVDisplayLinkSetOutputCallback (displayLink, &displayLinkCallback, this);
-        CVDisplayLinkStart (displayLink);
+        cvDisplayLinkWrapper = std::make_unique<CVDisplayLinkWrapper> (*this);
+        cvDisplayLinkWrapper->updateActiveDisplay();
+        nativeContext->setNominalVideoRefreshPeriodS (cvDisplayLinkWrapper->getNominalVideoRefreshPeriodS());
        #endif
 
         return true;
@@ -547,17 +599,14 @@ public:
     void shutdownOnThread()
     {
        #if JUCE_MAC
-        CVDisplayLinkStop (displayLink);
-        CVDisplayLinkRelease (displayLink);
+        cvDisplayLinkWrapper = nullptr;
        #endif
 
         if (context.renderer != nullptr)
             context.renderer->openGLContextClosing();
 
-       #if JUCE_OPENGL3
         if (vertexArrayObject != 0)
             context.extensions.glDeleteVertexArrays (1, &vertexArrayObject);
-       #endif
 
         associatedObjectNames.clear();
         associatedObjects.clear();
@@ -638,7 +687,7 @@ public:
         }
         else
         {
-            jassertfalse; // you called execute AFTER you detached your openglcontext
+            jassertfalse; // you called execute AFTER you detached your OpenGLContext
         }
     }
 
@@ -655,30 +704,108 @@ public:
     OpenGLContext& context;
     Component& component;
 
+    Version openGLVersion;
     OpenGLFrameBuffer cachedImageFrameBuffer;
     RectangleList<int> validArea;
     Rectangle<int> viewportArea, lastScreenBounds;
     double scale = 1.0;
     AffineTransform transform;
-   #if JUCE_OPENGL3
     GLuint vertexArrayObject = 0;
-   #endif
 
     StringArray associatedObjectNames;
     ReferenceCountedArray<ReferenceCountedObject> associatedObjects;
 
-    WaitableEvent canPaintNowFlag, finishedPaintingFlag, repaintEvent;
+    WaitableEvent canPaintNowFlag, finishedPaintingFlag, repaintEvent { true };
    #if JUCE_OPENGL_ES
     bool shadersAvailable = true;
    #else
     bool shadersAvailable = false;
    #endif
+    bool textureNpotSupported = false;
     std::atomic<bool> hasInitialised { false }, needsUpdate { true }, destroying { false };
     uint32 lastMMLockReleaseTime = 0;
 
    #if JUCE_MAC
-    CVDisplayLinkRef displayLink;
+    NSView* getCurrentView() const
+    {
+        if (auto* peer = component.getPeer())
+            return static_cast<NSView*> (peer->getNativeHandle());
+
+        return nullptr;
+    }
+
+    NSScreen* getCurrentScreen() const
+    {
+        if (auto* view = getCurrentView())
+            if (auto* window = [view window])
+                return [window screen];
+
+        return nullptr;
+    }
+
+    struct CVDisplayLinkWrapper
+    {
+        explicit CVDisplayLinkWrapper (CachedImage& cachedImageIn)
+            : cachedImage (cachedImageIn),
+              continuousRepaint (cachedImageIn.context.continuousRepaint.load())
+        {
+            CVDisplayLinkCreateWithActiveCGDisplays (&displayLink);
+            CVDisplayLinkSetOutputCallback (displayLink, &displayLinkCallback, this);
+            CVDisplayLinkStart (displayLink);
+        }
+
+        double getNominalVideoRefreshPeriodS() const
+        {
+            const auto nominalVideoRefreshPeriod = CVDisplayLinkGetNominalOutputVideoRefreshPeriod (displayLink);
+
+            if ((nominalVideoRefreshPeriod.flags & kCVTimeIsIndefinite) == 0)
+                return (double) nominalVideoRefreshPeriod.timeValue / (double) nominalVideoRefreshPeriod.timeScale;
+
+            return 0.0;
+        }
+
+        /*  Returns true if updated, or false otherwise. */
+        bool updateActiveDisplay()
+        {
+            auto* oldScreen = std::exchange (currentScreen, cachedImage.getCurrentScreen());
+
+            if (oldScreen == currentScreen)
+                return false;
+
+            for (NSScreen* screen in [NSScreen screens])
+                if (screen == currentScreen)
+                    if (NSNumber* number = [[screen deviceDescription] objectForKey: @"NSScreenNumber"])
+                        CVDisplayLinkSetCurrentCGDisplay (displayLink, [number unsignedIntValue]);
+
+            return true;
+        }
+
+        ~CVDisplayLinkWrapper()
+        {
+            CVDisplayLinkStop (displayLink);
+            CVDisplayLinkRelease (displayLink);
+        }
+
+        static CVReturn displayLinkCallback (CVDisplayLinkRef, const CVTimeStamp*, const CVTimeStamp*,
+                                             CVOptionFlags, CVOptionFlags*, void* displayLinkContext)
+        {
+            auto* self = reinterpret_cast<CVDisplayLinkWrapper*> (displayLinkContext);
+
+            if (self->continuousRepaint)
+                self->cachedImage.repaintEvent.signal();
+
+            return kCVReturnSuccess;
+        }
+
+        CachedImage& cachedImage;
+        const bool continuousRepaint;
+        CVDisplayLinkRef displayLink;
+        NSScreen* currentScreen = nullptr;
+    };
+
+    std::unique_ptr<CVDisplayLinkWrapper> cvDisplayLinkWrapper;
    #endif
+
     std::unique_ptr<ThreadPool> renderThread;
     ReferenceCountedArray<OpenGLContext::AsyncWorker, CriticalSection> workQueue;
     MessageManager::Lock messageManagerLock;
@@ -882,6 +1009,15 @@ void OpenGLContext::setComponentPaintingEnabled (bool shouldPaintComponent) noex
 void OpenGLContext::setContinuousRepainting (bool shouldContinuouslyRepaint) noexcept
 {
     continuousRepaint = shouldContinuouslyRepaint;
+
+    #if JUCE_MAC
+     if (auto* component = getTargetComponent())
+     {
+         detach();
+         attachment.reset (new Attachment (*this, *component));
+     }
+    #endif
+
     triggerRepaint();
 }
 
@@ -1040,6 +1176,12 @@ bool OpenGLContext::areShadersAvailable() const
     return c != nullptr && c->shadersAvailable;
 }
 
+bool OpenGLContext::isTextureNpotSupported() const
+{
+    auto* c = getCachedImage();
+    return c != nullptr && c->textureNpotSupported;
+}
+
 ReferenceCountedObject* OpenGLContext::getAssociatedObject (const char* name) const
 {
     jassert (name != nullptr);
