@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -24,9 +23,14 @@
   ==============================================================================
 */
 
+#if (defined (__IPHONE_10_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_10_0)
+ JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wdeprecated-declarations")
+ #define JUCE_USE_NEW_CAMERA_API 1
+#endif
+
 struct CameraDevice::Pimpl
 {
-    using InternalOpenCameraResultCallback = std::function<void(const String& /*cameraId*/, const String& /*error*/)>;
+    using InternalOpenCameraResultCallback = std::function<void (const String& /*cameraId*/, const String& /*error*/)>;
 
     Pimpl (CameraDevice& ownerToUse, const String& cameraIdToUse, int /*index*/,
            int /*minWidth*/, int /*minHeight*/, int /*maxWidth*/, int /*maxHeight*/,
@@ -55,9 +59,7 @@ struct CameraDevice::Pimpl
          {
              // Access to video is required for camera to work,
              // black images will be produced otherwise!
-             jassert (granted);
-
-             ignoreUnused (granted);
+             jassertquiet (granted);
          }];
 
         [AVCaptureDevice requestAccessForMediaType: AVMediaTypeAudio
@@ -65,9 +67,7 @@ struct CameraDevice::Pimpl
          {
              // Access to audio is required for camera to work,
              // silence will be produced otherwise!
-             jassert (granted);
-
-             ignoreUnused (granted);
+             jassertquiet (granted);
          }];
 
         captureSession.startSessionForDeviceWithId (cameraId);
@@ -75,7 +75,7 @@ struct CameraDevice::Pimpl
 
     bool openedOk() const noexcept { return captureSession.openedOk(); }
 
-    void takeStillPicture (std::function<void(const Image&)> pictureTakenCallbackToUse)
+    void takeStillPicture (std::function<void (const Image&)> pictureTakenCallbackToUse)
     {
         if (pictureTakenCallbackToUse == nullptr)
         {
@@ -141,18 +141,18 @@ struct CameraDevice::Pimpl
 private:
     static NSArray<AVCaptureDevice*>* getDevices()
     {
-       #if defined (__IPHONE_10_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_10_0
-        if (iosVersion.major >= 10)
+       #if JUCE_USE_NEW_CAMERA_API
+        if (@available (iOS 10.0, *))
         {
             std::unique_ptr<NSMutableArray<AVCaptureDeviceType>, NSObjectDeleter> deviceTypes ([[NSMutableArray alloc] initWithCapacity: 2]);
 
             [deviceTypes.get() addObject: AVCaptureDeviceTypeBuiltInWideAngleCamera];
             [deviceTypes.get() addObject: AVCaptureDeviceTypeBuiltInTelephotoCamera];
 
-            if ((iosVersion.major == 10 && iosVersion.minor >= 2) || iosVersion.major >= 11)
+            if (@available (iOS 10.2, *))
                 [deviceTypes.get() addObject: AVCaptureDeviceTypeBuiltInDualCamera];
 
-            if ((iosVersion.major == 11 && iosVersion.minor >= 1) || iosVersion.major >= 12)
+            if (@available (iOS 11.1, *))
                 [deviceTypes.get() addObject: AVCaptureDeviceTypeBuiltInTrueDepthCamera];
 
             auto discoverySession = [AVCaptureDeviceDiscoverySession discoverySessionWithDeviceTypes: deviceTypes.get()
@@ -207,8 +207,8 @@ private:
         JUCE_CAMERA_LOG ("Supports custom exposure: " + String ((int)[device isExposureModeSupported: AVCaptureExposureModeCustom]));
         JUCE_CAMERA_LOG ("Supports point of interest exposure: " + String ((int)device.exposurePointOfInterestSupported));
 
-       #if defined (__IPHONE_10_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_10_0
-        if (iosVersion.major >= 10)
+       #if JUCE_USE_NEW_CAMERA_API
+        if (@available (iOS 10.0, *))
         {
             JUCE_CAMERA_LOG ("Device type: " + nsStringToJuce (device.deviceType));
             JUCE_CAMERA_LOG ("Locking focus with custom lens position supported: " + String ((int)device.lockingFocusWithCustomLensPositionSupported));
@@ -216,7 +216,7 @@ private:
        #endif
 
        #if defined (__IPHONE_11_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_11_0
-        if (iosVersion.major >= 11)
+        if (@available (iOS 11.0, *))
         {
             JUCE_CAMERA_LOG ("Min available video zoom factor: " + String (device.minAvailableVideoZoomFactor));
             JUCE_CAMERA_LOG ("Max available video zoom factor: " + String (device.maxAvailableVideoZoomFactor));
@@ -238,19 +238,24 @@ private:
     {
         JUCE_CAMERA_LOG ("Media type: " + nsStringToJuce (format.mediaType));
 
-        String colourSpaces;
-
-        for (NSNumber* number in format.supportedColorSpaces)
+       #if JUCE_USE_NEW_CAMERA_API
+        if (@available (iOS 10.0, *))
         {
-            switch ([number intValue])
-            {
-                case AVCaptureColorSpace_sRGB:   colourSpaces << "sRGB ";  break;
-                case AVCaptureColorSpace_P3_D65: colourSpaces << "P3_D65 "; break;
-                default: break;
-            }
-        }
+            String colourSpaces;
 
-        JUCE_CAMERA_LOG ("Supported colour spaces: " + colourSpaces);
+            for (NSNumber* number in format.supportedColorSpaces)
+            {
+                switch ([number intValue])
+                {
+                    case AVCaptureColorSpace_sRGB:   colourSpaces << "sRGB ";  break;
+                    case AVCaptureColorSpace_P3_D65: colourSpaces << "P3_D65 "; break;
+                    default: break;
+                }
+            }
+
+            JUCE_CAMERA_LOG ("Supported colour spaces: " + colourSpaces);
+        }
+       #endif
 
         JUCE_CAMERA_LOG ("Video field of view: " + String (format.videoFieldOfView));
         JUCE_CAMERA_LOG ("Video max zoom factor: " + String (format.videoMaxZoomFactor));
@@ -262,34 +267,30 @@ private:
             videoFrameRateRangesString << frameRateRangeToString (range);
         JUCE_CAMERA_LOG (videoFrameRateRangesString);
 
-        JUCE_CAMERA_LOG ("Video binned: " + String (int(format.videoBinned)));
+        JUCE_CAMERA_LOG ("Video binned: " + String (int (format.videoBinned)));
 
-       #if defined (__IPHONE_8_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_8_0
-        if (iosVersion.major >= 8)
+        JUCE_CAMERA_LOG ("Video HDR supported: " + String (int (format.videoHDRSupported)));
+        JUCE_CAMERA_LOG ("High resolution still image dimensions: " + getHighResStillImgDimensionsString (format.highResolutionStillImageDimensions));
+        JUCE_CAMERA_LOG ("Min ISO: " + String (format.minISO));
+        JUCE_CAMERA_LOG ("Max ISO: " + String (format.maxISO));
+        JUCE_CAMERA_LOG ("Min exposure duration: " + cmTimeToString (format.minExposureDuration));
+
+        String autoFocusSystemString;
+        switch (format.autoFocusSystem)
         {
-            JUCE_CAMERA_LOG ("Video HDR supported: " + String (int (format.videoHDRSupported)));
-            JUCE_CAMERA_LOG ("High resolution still image dimensions: " + getHighResStillImgDimensionsString (format.highResolutionStillImageDimensions));
-            JUCE_CAMERA_LOG ("Min ISO: " + String (format.minISO));
-            JUCE_CAMERA_LOG ("Max ISO: " + String (format.maxISO));
-            JUCE_CAMERA_LOG ("Min exposure duration: " + cmTimeToString (format.minExposureDuration));
-
-            String autoFocusSystemString;
-            switch (format.autoFocusSystem)
-            {
-                case AVCaptureAutoFocusSystemPhaseDetection:    autoFocusSystemString = "PhaseDetection";    break;
-                case AVCaptureAutoFocusSystemContrastDetection: autoFocusSystemString = "ContrastDetection"; break;
-                default: autoFocusSystemString = "None";
-            }
-            JUCE_CAMERA_LOG ("Auto focus system: " + autoFocusSystemString);
-
-            JUCE_CAMERA_LOG ("Standard (iOS 5.0) video stabilization supported: " + String ((int) [format isVideoStabilizationModeSupported: AVCaptureVideoStabilizationModeStandard]));
-            JUCE_CAMERA_LOG ("Cinematic video stabilization supported: " + String ((int) [format isVideoStabilizationModeSupported: AVCaptureVideoStabilizationModeCinematic]));
-            JUCE_CAMERA_LOG ("Auto video stabilization supported: " + String ((int) [format isVideoStabilizationModeSupported: AVCaptureVideoStabilizationModeAuto]));
+            case AVCaptureAutoFocusSystemPhaseDetection:    autoFocusSystemString = "PhaseDetection";    break;
+            case AVCaptureAutoFocusSystemContrastDetection: autoFocusSystemString = "ContrastDetection"; break;
+            case AVCaptureAutoFocusSystemNone:
+            default:                                        autoFocusSystemString = "None";
         }
-       #endif
+        JUCE_CAMERA_LOG ("Auto focus system: " + autoFocusSystemString);
+
+        JUCE_CAMERA_LOG ("Standard (iOS 5.0) video stabilization supported: " + String ((int) [format isVideoStabilizationModeSupported: AVCaptureVideoStabilizationModeStandard]));
+        JUCE_CAMERA_LOG ("Cinematic video stabilization supported: " + String ((int) [format isVideoStabilizationModeSupported: AVCaptureVideoStabilizationModeCinematic]));
+        JUCE_CAMERA_LOG ("Auto video stabilization supported: " + String ((int) [format isVideoStabilizationModeSupported: AVCaptureVideoStabilizationModeAuto]));
 
        #if defined (__IPHONE_11_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_11_0
-        if (iosVersion.major >= 11)
+        if (@available (iOS 11.0, *))
         {
             JUCE_CAMERA_LOG ("Min zoom factor for depth data delivery: " + String (format.videoMinZoomFactorForDepthDataDelivery));
             JUCE_CAMERA_LOG ("Max zoom factor for depth data delivery: " + String (format.videoMaxZoomFactorForDepthDataDelivery));
@@ -304,11 +305,8 @@ private:
 
     static String cmTimeToString (CMTime time)
     {
-        CFStringRef timeDesc = CMTimeCopyDescription (nullptr, time);
-        String result = String::fromCFString (timeDesc);
-
-        CFRelease (timeDesc);
-        return result;
+        CFUniquePtr<CFStringRef> timeDesc (CMTimeCopyDescription (nullptr, time));
+        return String::fromCFString (timeDesc.get());
     }
 
     static String frameRateRangeToString (AVFrameRateRange* range)
@@ -338,8 +336,7 @@ private:
             delegate.reset ([cls.createInstance() init]);
             SessionDelegateClass::setOwner (delegate.get(), this);
 
-          #pragma clang diagnostic push
-          #pragma clang diagnostic ignored "-Wundeclared-selector"
+            JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wundeclared-selector")
             [[NSNotificationCenter defaultCenter] addObserver: delegate.get()
                                                      selector: @selector (sessionDidStartRunning:)
                                                          name: AVCaptureSessionDidStartRunningNotification
@@ -351,7 +348,7 @@ private:
                                                        object: captureSession.get()];
 
             [[NSNotificationCenter defaultCenter] addObserver: delegate.get()
-                                                     selector: @selector (sessionRuntimeError:)
+                                                     selector: @selector (runtimeError:)
                                                          name: AVCaptureSessionRuntimeErrorNotification
                                                        object: captureSession.get()];
 
@@ -364,7 +361,7 @@ private:
                                                      selector: @selector (sessionInterruptionEnded:)
                                                          name: AVCaptureSessionInterruptionEndedNotification
                                                        object: captureSession.get()];
-           #pragma clang diagnostic pop
+            JUCE_END_IGNORE_WARNINGS_GCC_LIKE
 
             dispatch_async (captureSessionQueue,^
                             {
@@ -411,9 +408,7 @@ private:
 
                                 if (error.isNotEmpty())
                                 {
-                                    WeakReference<CaptureSession> weakRef (this);
-
-                                    MessageManager::callAsync ([weakRef, error]() mutable
+                                    MessageManager::callAsync ([weakRef = WeakReference<CaptureSession> { this }, error]() mutable
                                     {
                                         if (weakRef != nullptr)
                                             weakRef->owner.cameraOpenCallback ({}, error);
@@ -427,9 +422,7 @@ private:
 
                                 if (error.isNotEmpty())
                                 {
-                                    WeakReference<CaptureSession> weakRef (this);
-
-                                    MessageManager::callAsync ([weakRef, error]() mutable
+                                    MessageManager::callAsync ([weakRef = WeakReference<CaptureSession> { this }, error]() mutable
                                     {
                                         if (weakRef != nullptr)
                                             weakRef->owner.cameraOpenCallback ({}, error);
@@ -525,14 +518,13 @@ private:
         {
             SessionDelegateClass()  : ObjCClass<NSObject> ("SessionDelegateClass_")
             {
-               #pragma clang diagnostic push
-               #pragma clang diagnostic ignored "-Wundeclared-selector"
-                addMethod (@selector (sessionDidStartRunning:),   started,           "v@:@");
-                addMethod (@selector (sessionDidStopRunning:),    stopped,           "v@:@");
-                addMethod (@selector (sessionRuntimeError:),      runtimeError,      "v@:@");
-                addMethod (@selector (sessionWasInterrupted:),    interrupted,       "v@:@");
-                addMethod (@selector (sessionInterruptionEnded:), interruptionEnded, "v@:@");
-               #pragma clang diagnostic pop
+                JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wundeclared-selector")
+                addMethod (@selector (sessionDidStartRunning:),   started);
+                addMethod (@selector (sessionDidStopRunning:),    stopped);
+                addMethod (@selector (runtimeError:),             runtimeError);
+                addMethod (@selector (sessionWasInterrupted:),    interrupted);
+                addMethod (@selector (sessionInterruptionEnded:), interruptionEnded);
+                JUCE_END_IGNORE_WARNINGS_GCC_LIKE
 
                 addIvar<CaptureSession*> ("owner");
 
@@ -600,12 +592,14 @@ private:
                   captureOutput (createCaptureOutput()),
                   photoOutputDelegate (nullptr)
             {
-                if (Pimpl::getIOSVersion().major >= 10)
+               #if JUCE_USE_NEW_CAMERA_API
+                if (@available (iOS 10.0, *))
                 {
                     static PhotoOutputDelegateClass cls;
                     photoOutputDelegate.reset ([cls.createInstance() init]);
                     PhotoOutputDelegateClass::setOwner (photoOutputDelegate.get(), this);
                 }
+               #endif
 
                 captureSession.addOutputIfPossible (captureOutput);
             }
@@ -625,17 +619,20 @@ private:
 
                 if (auto* connection = findVideoConnection (captureOutput))
                 {
-                   #if defined (__IPHONE_10_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_10_0
-                    if (Pimpl::getIOSVersion().major >= 10 && [captureOutput isKindOfClass: [AVCapturePhotoOutput class]])
+                   #if JUCE_USE_NEW_CAMERA_API
+                    if (@available (iOS 10.0, *))
                     {
-                        auto* photoOutput = (AVCapturePhotoOutput*) captureOutput;
-                        auto outputConnection = [photoOutput connectionWithMediaType: AVMediaTypeVideo];
-                        outputConnection.videoOrientation = orientationToUse;
+                        if ([captureOutput isKindOfClass: [AVCapturePhotoOutput class]])
+                        {
+                            auto* photoOutput = (AVCapturePhotoOutput*) captureOutput;
+                            auto outputConnection = [photoOutput connectionWithMediaType: AVMediaTypeVideo];
+                            outputConnection.videoOrientation = orientationToUse;
 
-                        [photoOutput capturePhotoWithSettings: [AVCapturePhotoSettings photoSettings]
-                                                     delegate: id<AVCapturePhotoCaptureDelegate> (photoOutputDelegate.get())];
+                            [photoOutput capturePhotoWithSettings: [AVCapturePhotoSettings photoSettings]
+                                                         delegate: id<AVCapturePhotoCaptureDelegate> (photoOutputDelegate.get())];
 
-                        return;
+                            return;
+                        }
                     }
                    #endif
 
@@ -646,6 +643,8 @@ private:
                     [stillImageOutput captureStillImageAsynchronouslyFromConnection: connection completionHandler:
                          ^(CMSampleBufferRef imageSampleBuffer, NSError* error)
                          {
+                             takingPicture = false;
+
                              if (error != nil)
                              {
                                  JUCE_CAMERA_LOG ("Still picture capture failed, error: " + nsStringToJuce (error.localizedDescription));
@@ -659,7 +658,7 @@ private:
 
                              callListeners (image);
 
-                             MessageManager::callAsync ([this, image]() { notifyPictureTaken (image); });
+                             MessageManager::callAsync ([this, image] { notifyPictureTaken (image); });
                          }];
                 }
                 else
@@ -672,8 +671,8 @@ private:
         private:
             static AVCaptureOutput* createCaptureOutput()
             {
-               #if defined (__IPHONE_10_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_10_0
-                if (Pimpl::getIOSVersion().major >= 10)
+               #if JUCE_USE_NEW_CAMERA_API
+                if (@available (iOS 10.0, *))
                     return [AVCapturePhotoOutput new];
                #endif
 
@@ -682,58 +681,61 @@ private:
 
             static void printImageOutputDebugInfo (AVCaptureOutput* captureOutput)
             {
-               #if defined (__IPHONE_10_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_10_0
-                if (Pimpl::getIOSVersion().major >= 10 && [captureOutput isKindOfClass: [AVCapturePhotoOutput class]])
+               #if JUCE_USE_NEW_CAMERA_API
+                if (@available (iOS 10.0, *))
                 {
-                    auto* photoOutput = (AVCapturePhotoOutput*) captureOutput;
-
-                    String typesString;
-
-                    for (AVVideoCodecType type in photoOutput.availablePhotoCodecTypes)
-                        typesString << nsStringToJuce (type) << " ";
-
-                    JUCE_CAMERA_LOG ("Available image codec types: " + typesString);
-
-                    JUCE_CAMERA_LOG ("Still image stabilization supported: " + String ((int) photoOutput.stillImageStabilizationSupported));
-                    JUCE_CAMERA_LOG ("Dual camera fusion supported: " + String ((int) photoOutput.dualCameraFusionSupported));
-                    JUCE_CAMERA_LOG ("Supports flash: "      + String ((int) [photoOutput.supportedFlashModes containsObject: @(AVCaptureFlashModeOn)]));
-                    JUCE_CAMERA_LOG ("Supports auto flash: " + String ((int) [photoOutput.supportedFlashModes containsObject: @(AVCaptureFlashModeAuto)]));
-                    JUCE_CAMERA_LOG ("Max bracketed photo count: " + String (photoOutput.maxBracketedCapturePhotoCount));
-                    JUCE_CAMERA_LOG ("Lens stabilization during bracketed capture supported: " + String ((int) photoOutput.lensStabilizationDuringBracketedCaptureSupported));
-                    JUCE_CAMERA_LOG ("Live photo capture supported: " + String ((int) photoOutput.livePhotoCaptureSupported));
-
-
-                   #if defined (__IPHONE_11_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_11_0
-                    if (Pimpl::getIOSVersion().major >= 11)
+                    if ([captureOutput isKindOfClass: [AVCapturePhotoOutput class]])
                     {
-                        typesString.clear();
+                        auto* photoOutput = (AVCapturePhotoOutput*) captureOutput;
 
-                        for (AVFileType type in photoOutput.availablePhotoFileTypes)
+                        String typesString;
+
+                        for (id type in photoOutput.availablePhotoCodecTypes)
                             typesString << nsStringToJuce (type) << " ";
 
-                        JUCE_CAMERA_LOG ("Available photo file types: " + typesString);
+                        JUCE_CAMERA_LOG ("Available image codec types: " + typesString);
 
-                        typesString.clear();
+                        JUCE_CAMERA_LOG ("Still image stabilization supported: " + String ((int) photoOutput.stillImageStabilizationSupported));
+                        JUCE_CAMERA_LOG ("Dual camera fusion supported: " + String ((int) photoOutput.dualCameraFusionSupported));
+                        JUCE_CAMERA_LOG ("Supports flash: "      + String ((int) [photoOutput.supportedFlashModes containsObject: @(AVCaptureFlashModeOn)]));
+                        JUCE_CAMERA_LOG ("Supports auto flash: " + String ((int) [photoOutput.supportedFlashModes containsObject: @(AVCaptureFlashModeAuto)]));
+                        JUCE_CAMERA_LOG ("Max bracketed photo count: " + String (photoOutput.maxBracketedCapturePhotoCount));
+                        JUCE_CAMERA_LOG ("Lens stabilization during bracketed capture supported: " + String ((int) photoOutput.lensStabilizationDuringBracketedCaptureSupported));
+                        JUCE_CAMERA_LOG ("Live photo capture supported: " + String ((int) photoOutput.livePhotoCaptureSupported));
 
-                        for (AVFileType type in photoOutput.availableRawPhotoFileTypes)
-                            typesString << nsStringToJuce (type) << " ";
 
-                        JUCE_CAMERA_LOG ("Available RAW photo file types: " + typesString);
+                       #if defined (__IPHONE_11_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_11_0
+                        if (@available (iOS 11.0, *))
+                        {
+                            typesString.clear();
 
-                        typesString.clear();
+                            for (AVFileType type in photoOutput.availablePhotoFileTypes)
+                                typesString << nsStringToJuce (type) << " ";
 
-                        for (AVFileType type in photoOutput.availableLivePhotoVideoCodecTypes)
-                            typesString << nsStringToJuce (type) << " ";
+                            JUCE_CAMERA_LOG ("Available photo file types: " + typesString);
 
-                        JUCE_CAMERA_LOG ("Available live photo video codec types: " + typesString);
+                            typesString.clear();
 
-                        JUCE_CAMERA_LOG ("Dual camera dual photo delivery supported: " + String ((int) photoOutput.dualCameraDualPhotoDeliverySupported));
-                        JUCE_CAMERA_LOG ("Camera calibration data delivery supported: " + String ((int) photoOutput.cameraCalibrationDataDeliverySupported));
-                        JUCE_CAMERA_LOG ("Depth data delivery supported: " + String ((int) photoOutput.depthDataDeliverySupported));
+                            for (AVFileType type in photoOutput.availableRawPhotoFileTypes)
+                                typesString << nsStringToJuce (type) << " ";
+
+                            JUCE_CAMERA_LOG ("Available RAW photo file types: " + typesString);
+
+                            typesString.clear();
+
+                            for (AVFileType type in photoOutput.availableLivePhotoVideoCodecTypes)
+                                typesString << nsStringToJuce (type) << " ";
+
+                            JUCE_CAMERA_LOG ("Available live photo video codec types: " + typesString);
+
+                            JUCE_CAMERA_LOG ("Dual camera dual photo delivery supported: " + String ((int) photoOutput.dualCameraDualPhotoDeliverySupported));
+                            JUCE_CAMERA_LOG ("Camera calibration data delivery supported: " + String ((int) photoOutput.cameraCalibrationDataDeliverySupported));
+                            JUCE_CAMERA_LOG ("Depth data delivery supported: " + String ((int) photoOutput.depthDataDeliverySupported));
+                        }
+                       #endif
+
+                        return;
                     }
-                   #endif
-
-                    return;
                 }
                #endif
 
@@ -741,7 +743,7 @@ private:
 
                 String typesString;
 
-                for (AVVideoCodecType type in stillImageOutput.availableImageDataCodecTypes)
+                for (id type in stillImageOutput.availableImageDataCodecTypes)
                     typesString << nsStringToJuce (type) << " ";
 
                 JUCE_CAMERA_LOG ("Available image codec types: " + typesString);
@@ -763,20 +765,27 @@ private:
             }
 
             //==============================================================================
+           #if JUCE_USE_NEW_CAMERA_API
             class PhotoOutputDelegateClass : public ObjCClass<NSObject>
             {
             public:
                 PhotoOutputDelegateClass() : ObjCClass<NSObject> ("PhotoOutputDelegateClass_")
                 {
-                    addMethod (@selector (captureOutput:willBeginCaptureForResolvedSettings:),       willBeginCaptureForSettings, "v@:@@");
-                    addMethod (@selector (captureOutput:willCapturePhotoForResolvedSettings:),       willCaptureForSettings,      "v@:@@");
-                    addMethod (@selector (captureOutput:didCapturePhotoForResolvedSettings:),        didCaptureForSettings,       "v@:@@");
-                    addMethod (@selector (captureOutput:didFinishCaptureForResolvedSettings:error:), didFinishCaptureForSettings, "v@:@@@");
+                    addMethod (@selector (captureOutput:willBeginCaptureForResolvedSettings:),       willBeginCaptureForSettings);
+                    addMethod (@selector (captureOutput:willCapturePhotoForResolvedSettings:),       willCaptureForSettings);
+                    addMethod (@selector (captureOutput:didCapturePhotoForResolvedSettings:),        didCaptureForSettings);
+                    addMethod (@selector (captureOutput:didFinishCaptureForResolvedSettings:error:), didFinishCaptureForSettings);
 
-                    if (Pimpl::getIOSVersion().major >= 11)
-                        addMethod (@selector (captureOutput:didFinishProcessingPhoto:error:), didFinishProcessingPhoto, "v@:@@@");
+                   #if defined (__IPHONE_11_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_11_0
+                    if (@available (iOS 11.0, *))
+                    {
+                        addMethod (@selector (captureOutput:didFinishProcessingPhoto:error:), didFinishProcessingPhoto);
+                    }
                     else
-                        addMethod (@selector (captureOutput:didFinishProcessingPhotoSampleBuffer:previewPhotoSampleBuffer:resolvedSettings:bracketSettings:error:), didFinishProcessingPhotoSampleBuffer, "v@:@@@@@@");
+                   #endif
+                    {
+                        addMethod (@selector (captureOutput:didFinishProcessingPhotoSampleBuffer:previewPhotoSampleBuffer:resolvedSettings:bracketSettings:error:), didFinishProcessingPhotoSampleBuffer);
+                    }
 
                     addIvar<StillPictureTaker*> ("owner");
 
@@ -813,6 +822,8 @@ private:
 
                 static void didFinishProcessingPhoto (id self, SEL, AVCapturePhotoOutput*, AVCapturePhoto* capturePhoto, NSError* error)
                 {
+                    getOwner (self).takingPicture = false;
+
                     String errorString = error != nil ? nsStringToJuce (error.localizedDescription) : String();
                     ignoreUnused (errorString);
 
@@ -869,6 +880,10 @@ private:
                             CGContextScaleCTM (context, targetSize.height / origHeight, -targetSize.width / origWidth);
                             CGContextTranslateCTM (context, -targetSize.width, -targetSize.height);
                             break;
+                        case kCGImagePropertyOrientationUpMirrored:
+                        case kCGImagePropertyOrientationDownMirrored:
+                        case kCGImagePropertyOrientationLeftMirrored:
+                        case kCGImagePropertyOrientationRightMirrored:
                         default:
                             // Not implemented.
                             jassertfalse;
@@ -912,6 +927,8 @@ private:
                                                                   AVCaptureResolvedPhotoSettings*, AVCaptureBracketedStillImageSettings*,
                                                                   NSError* error)
                 {
+                    getOwner (self).takingPicture = false;
+
                     String errorString = error != nil ? nsStringToJuce (error.localizedDescription) : String();
                     ignoreUnused (errorString);
 
@@ -954,6 +971,7 @@ private:
                     }
                 }
             };
+           #endif
 
             //==============================================================================
             void callListeners (const Image& image)
@@ -963,8 +981,6 @@ private:
 
             void notifyPictureTaken (const Image& image)
             {
-                takingPicture = false;
-
                 captureSession.notifyPictureTaken (image);
             }
 
@@ -1006,8 +1022,10 @@ private:
 
             void startRecording (const File& file, AVCaptureVideoOrientation orientationToUse)
             {
-                if (Pimpl::getIOSVersion().major >= 10)
+               #if JUCE_USE_NEW_CAMERA_API
+                if (@available (iOS 10.0, *))
                     printVideoOutputDebugInfo (movieFileOutput);
+               #endif
 
                 auto url = [NSURL fileURLWithPath: juceStringToNS (file.getFullPathName())
                                       isDirectory: NO];
@@ -1036,7 +1054,7 @@ private:
                 JUCE_CAMERA_LOG ("Available video codec types:");
 
                #if JUCE_CAMERA_LOG_ENABLED
-                for (AVVideoCodecType type in output.availableVideoCodecTypes)
+                for (id type in output.availableVideoCodecTypes)
                     JUCE_CAMERA_LOG (nsStringToJuce (type));
                #endif
 
@@ -1053,8 +1071,8 @@ private:
             {
                 FileOutputRecordingDelegateClass()  : ObjCClass<NSObject<AVCaptureFileOutputRecordingDelegate>> ("FileOutputRecordingDelegateClass_")
                 {
-                    addMethod (@selector (captureOutput:didStartRecordingToOutputFileAtURL:fromConnections:),        started, "v@:@@@");
-                    addMethod (@selector (captureOutput:didFinishRecordingToOutputFileAtURL:fromConnections:error:), stopped, "v@:@@@@");
+                    addMethod (@selector (captureOutput:didStartRecordingToOutputFileAtURL:fromConnections:),        started);
+                    addMethod (@selector (captureOutput:didFinishRecordingToOutputFileAtURL:fromConnections:error:), stopped);
 
                     addIvar<VideoRecorder*> ("owner");
 
@@ -1190,6 +1208,9 @@ private:
     {
         const ScopedLock sl (listenerLock);
         listeners.call ([=] (Listener& l) { l.imageReceived (image); });
+
+        if (listeners.size() == 1)
+            triggerStillPictureCapture();
     }
 
     void notifyPictureTaken (const Image& image)
@@ -1214,38 +1235,18 @@ private:
     CriticalSection listenerLock;
     ListenerList<Listener> listeners;
 
-    std::function<void(const Image&)> pictureTakenCallback;
+    std::function<void (const Image&)> pictureTakenCallback;
 
     CaptureSession captureSession;
 
     bool notifiedOfCameraOpening = false;
 
-    //==============================================================================
-    struct IOSVersion
-    {
-        int major;
-        int minor;
-    };
-
-    static IOSVersion getIOSVersion()
-    {
-        auto processInfo = [NSProcessInfo processInfo];
-
-        if (! [processInfo respondsToSelector: @selector (operatingSystemVersion)])
-            return {7, 0};   // Below 8.0 in fact, but only care that it's below 8
-
-        return { (int)[processInfo operatingSystemVersion].majorVersion,
-                 (int)[processInfo operatingSystemVersion].minorVersion };
-    }
-
-    static IOSVersion iosVersion;
-
     friend struct CameraDevice::ViewerComponent;
 
+    //==============================================================================
     JUCE_DECLARE_NON_COPYABLE (Pimpl)
 };
 
-CameraDevice::Pimpl::IOSVersion CameraDevice::Pimpl::iosVersion = CameraDevice::Pimpl::getIOSVersion();
 int CameraDevice::Pimpl::CaptureSession::numCaptureSessions = 0;
 
 //==============================================================================
@@ -1256,7 +1257,7 @@ struct CameraDevice::ViewerComponent  : public UIViewComponent
     {
         JuceCameraDeviceViewerClass()  : ObjCClass<UIView> ("JuceCameraDeviceViewerClass_")
         {
-            addMethod (@selector (layoutSubviews), layoutSubviews, "v@:");
+            addMethod (@selector (layoutSubviews), layoutSubviews);
 
             registerClass();
         }
@@ -1264,7 +1265,7 @@ struct CameraDevice::ViewerComponent  : public UIViewComponent
     private:
         static void layoutSubviews (id self, SEL)
         {
-            sendSuperclassMessage (self, @selector (layoutSubviews));
+            sendSuperclassMessage<void> (self, @selector (layoutSubviews));
 
             UIView* asUIView = (UIView*) self;
 
@@ -1329,3 +1330,7 @@ String CameraDevice::getFileExtension()
 {
     return ".mov";
 }
+
+#if JUCE_USE_NEW_CAMERA_API
+ JUCE_END_IGNORE_WARNINGS_GCC_LIKE
+#endif
